/***************************************************************************
                          kcdlabel.cpp  -  description
                             -------------------
    begin                : jeu mar 23 20:03:45 CET 2000
    copyright            : (C) 2000 by Pascal 'PeP' Panneels
    email                : pp@bvdep.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kcdlabel.h"

KCDLabel::KCDLabel(const char *name) : KTopLevelWidget(name)
{
	//---------------------------------------------------
	// setup the background color to white and the title
	//---------------------------------------------------
	setBackgroundColor(white);
	setCaption("KCD-Label");
	setBackgroundMode ( NoBackground );

	//-------------------------------------------	
	// create status bar
	//-------------------------------------------
	statusbar = new KStatusBar(this);
	setStatusBar( statusbar );
	statusbar->setInsertOrder(KStatusBar::RightToLeft);
	statusbar->insertItem( i18n("Page 0    "), STATUS_PAGE );
	statusbar->insertItem( " (100,100)", STATUS_MOUSEPOS);
	statusbar->insertItem( "KCDLabel v0.1 Pascal 'PeP' Panneels", STATUS_MSG );	
	statusbar->setAlignment( STATUS_PAGE, KStatusBar::Right);
	statusbar->setAlignment( STATUS_MOUSEPOS, KStatusBar::Right);
	
	//-------------------------------------------	
	// create the Popup menus and the main menu
	//-------------------------------------------
	filemenu = new QPopupMenu;
	filemenu->insertItem(i18n("&New"), this, SLOT(slotFileNew()));
	filemenu->insertItem(i18n("&Open..."), this, SLOT(slotFileOpen()));
	filemenu->insertItem(i18n("&Save..."), this, SLOT(slotFileSave()));
	filemenu->insertSeparator();
	filemenu->insertItem(i18n("&Print..."), this, SLOT(slotFilePrint()));
	filemenu->insertItem(i18n("Print set&up..."), this, SLOT(slotFilePrintSetup()));
	filemenu->insertSeparator();
	filemenu->insertItem(i18n("&Quit"), qApp, SLOT(quit()));

	bodymenu = new QPopupMenu;
	bodymenu->insertItem(i18n("&Select source"), this, SLOT(slotBodySelectSource()));
	bodymenu->insertItem(i18n("&Read content"), this, SLOT(slotBodyReadCD()));
	bodymenu->insertSeparator();
		columnmenu = new QPopupMenu;
		columnmenu->insertItem(i18n("&None"), COLUMN_0);
		columnmenu->insertItem(i18n("&1"), COLUMN_1);
		columnmenu->insertItem(i18n("&2"), COLUMN_2);
		columnmenu->insertItem(i18n("&3"), COLUMN_3);
		columnmenu->insertItem(i18n("&4"), COLUMN_4);
		columnmenu->insertItem(i18n("&5"), COLUMN_5);
		columnmenu->insertItem(i18n("&6"), COLUMN_6);
		QObject::connect(columnmenu, SIGNAL(activated(int)), this, SLOT(slotNumberColumn(int)) );
		columnmenu->setItemChecked(COLUMN_0, true);
	bodymenu->insertItem(i18n("Number of &columns"), columnmenu);
		levelmenu = new QPopupMenu;
		levelmenu->insertItem(i18n("&1"), LEVEL_1);
		levelmenu->insertItem(i18n("&2"), LEVEL_2);
		levelmenu->insertItem(i18n("&3"), LEVEL_3);
		levelmenu->insertItem(i18n("&4"), LEVEL_4);
		levelmenu->insertItem(i18n("&All"), LEVEL_ALL);
		QObject::connect(levelmenu, SIGNAL(activated(int)), this, SLOT(slotNumberLevel(int)));
		levelmenu->setItemChecked(LEVEL_ALL, true);
	bodymenu->insertItem(i18n("Number of &directory levels"), levelmenu);
	bodymenu->insertItem(i18n("&Font to use"), this, SLOT(slotBodySelectFont()));
	bodymenu->insertSeparator();
	bodymenu->insertItem(i18n("&Lined"), BODY_LINE);
	bodymenu->setItemChecked(BODY_LINE, true);
	QObject::connect(bodymenu, SIGNAL(activated(int)), this, SLOT(slotBodyLined(int)) );
	bodymenu->insertItem(i18n("Draw &tree lines"), TREE_LINE);
	bodymenu->setItemChecked(TREE_LINE, true);
	QObject::connect(bodymenu, SIGNAL(activated(int)), this, SLOT(slotBodyDrawTree(int)) );
    bodymenu->insertItem(i18n("&Page number"), DISPLAY_PAGENUMBER);
	QObject::connect(bodymenu, SIGNAL(activated(int)), this, SLOT(slotBodyPageNumber(int)) );
	bodymenu->insertSeparator();
	bodymenu->insertItem(i18n("Body &layout"), this, SLOT(slotBodyLayout()));
								
	slidemenu = new QPopupMenu;
	slidemenu->insertItem(i18n("&Edit slide text"), this, SLOT(slotSlideEditText()));
    	alignmenu = new QPopupMenu;
    	alignmenu->insertItem(i18n("&Left"), ALIGN_LEFT);
    	alignmenu->insertItem(i18n("&Center"), ALIGN_CENTER);
    	alignmenu->insertItem(i18n("&Right"), ALIGN_RIGHT);
        alignmenu->setItemChecked(ALIGN_LEFT, true);
    	QObject::connect(alignmenu, SIGNAL(activated(int)), this, SLOT(slotAlignMenu(int)));
    slidemenu->insertItem(i18n("&Align"), alignmenu);
	slidemenu->insertItem(i18n("Transparent Background"), SLIDE_BACKGROUND);
	slidemenu->setItemChecked(SLIDE_BACKGROUND, false);
	QObject::connect(slidemenu, SIGNAL(activated(int)), this, SLOT(slotImageTransparent(int)) );

	imagemenu = new QPopupMenu;
	imagemenu->insertItem(i18n("Insert..."), this, SLOT(slotImageInsert()));
	imagemenu->insertItem(i18n("Delete"), this, SLOT(slotObjectDelete()));
	imagemenu->insertSeparator();
	imagemenu->insertItem(i18n("Resize to full page"), this, SLOT(slotImageResizeFullPage()));
	imagemenu->insertItem(i18n("Resize to ..."), this, SLOT(slotImageResize()));
	imagemenu->insertSeparator();
    	alignimagemenu = new QPopupMenu;
    	alignimagemenu->insertItem(i18n("&Center of page"), this, SLOT(slotImageAlignCenter()));
    imagemenu->insertItem(i18n("Align"), alignimagemenu);
    	orderimagemenu = new QPopupMenu;
    	orderimagemenu->insertItem(i18n("Increase"), this, SLOT(slotImageIncreaseOrder()));
		orderimagemenu->insertItem(i18n("Decrease"), this, SLOT(slotImageDecreaseOrder()));
    imagemenu->insertItem(i18n("Order..."), orderimagemenu);
    // set some ids
    imagemenu->setId(1, IMAGE_MENU_DELETE);
	imagemenu->setId(3, IMAGE_MENU_RESIZEFULL);
	imagemenu->setId(4, IMAGE_MENU_RESIZE);
	imagemenu->setId(6, IMAGE_MENU_ALIGN);
	imagemenu->setId(7, IMAGE_MENU_ORDER);

	stringmenu = new QPopupMenu;
	stringmenu->insertItem(i18n("Insert..."), this, SLOT(slotStringInsert()));
	stringmenu->insertItem(i18n("Delete"), this, SLOT(slotObjectDelete()));
    stringmenu->insertItem(i18n("Modify"), this, SLOT(slotStringModify()));
    stringmenu->insertSeparator();
    	orderstringmenu = new QPopupMenu;
    	orderstringmenu->insertItem(i18n("Increase"), this, SLOT(slotStringIncreaseOrder()));
		orderstringmenu->insertItem(i18n("Decrease"), this, SLOT(slotStringDecreaseOrder()));
    stringmenu->insertItem(i18n("Order..."), orderstringmenu);
    // set some ids
    stringmenu->setId(1, STRING_MENU_DELETE);
	stringmenu->setId(2, STRING_MENU_MODIFY);
	stringmenu->setId(4, STRING_MENU_ORDER);
    			
	pagemenu = new QPopupMenu;
	pagemenu->insertItem(i18n("Go to &front page"), this, SLOT(slotPageFront()));
	pagemenu->insertItem(i18n("Go to &back page"), this, SLOT(slotPageBack()));
	pagemenu->insertItem(i18n("Go to &page number..."), this, SLOT(slotPageNumber()));
	
	helpmenu = new QPopupMenu;
	helpmenu->insertItem(i18n("Manual"), this, SLOT(slotHelpManual()));
	helpmenu->insertItem(i18n("About..."), this, SLOT(slotHelpAbout()));
	
	menubar = new QMenuBar( this );
	menubar->insertItem(i18n("&File"), filemenu);
	menubar->insertItem(i18n("&Body"), bodymenu);
	menubar->insertItem(i18n("&Slide"), slidemenu);
	menubar->insertItem(i18n("&Image"), imagemenu);
	menubar->insertItem(i18n("S&tring"), stringmenu);	
	menubar->insertItem(i18n("&Page"), pagemenu);
	menubar->insertSeparator();
	menubar->insertItem(i18n("&Help"), helpmenu);
	
	//---------------------------------------------------
	// define popup menus activated by right click
	//---------------------------------------------------
	ImagePopupMenu = new QPopupMenu;
	ImagePopupMenu->insertItem(i18n("Resize to full page"), this, SLOT(slotImageResizeFullPage()));
	ImagePopupMenu->insertItem(i18n("Resize to ..."), this, SLOT(slotImageResize()));
	ImagePopupMenu->insertItem(i18n("Delete"), this, SLOT(slotObjectDelete()));
    	orderImagePopupMenu = new QPopupMenu;
    	orderImagePopupMenu->insertItem(i18n("Increase"), this, SLOT(slotImageIncreaseOrder()));
		orderImagePopupMenu->insertItem(i18n("Decrease"), this, SLOT(slotImageDecreaseOrder()));
 	ImagePopupMenu->insertItem(i18n("Order..."), orderImagePopupMenu);
    	alignImagePopupMenu = new QPopupMenu;
    	alignImagePopupMenu->insertItem(i18n("&Center"), this, SLOT(slotImageAlignCenter()));
 	ImagePopupMenu->insertItem(i18n("Align..."), alignImagePopupMenu);
 	 	
	GlobalPopupMenu = new QPopupMenu;
	GlobalPopupMenu->insertItem(i18n("Insert image..."), this, SLOT(slotImageInsert()));
    GlobalPopupMenu->insertItem(i18n("Insert string..."), this, SLOT(slotStringInsert()));

    StringPopupMenu = new QPopupMenu;
    StringPopupMenu->insertItem(i18n("Delete"), this, SLOT(slotObjectDelete()));
    StringPopupMenu->insertItem(i18n("Modify"), this, SLOT(slotStringModify()));

	//---------------------------------------------------
	// setup the printing stuff
	//---------------------------------------------------
	Printing = false;
	Printer = new QPrinter;
  	Printer->setOrientation( QPrinter::Portrait );

	//---------------------------------------------------
	// setup misc stuff
	//---------------------------------------------------
  	ClickedPos = QPoint(10,10);
  	CurrentPageToDisplay = 0;
  	Slide_Align = AlignLeft;
  	AlignChecked = ALIGN_LEFT;
	ObjectSelected = NULL;
	InitialPosForMove = QPoint(0,0);
	bufferpixmap.resize(size());
	SlideTransparentBackground=false;
	for (int i=0;i<MAX_PAGES;i++)
		FrontPages.push_back(KCoverPage());
	myDirTree = new KDirTree("/mnt/cdrom");
	strcpy(SourceCD,"/mnt/cdrom");
    Levels = LEVEL_ALL;		// levels of directory  --> 0 means all
    Columns = COLUMN_0;		// no columns
	BodyPage = 2;
    BodyFont = new QFont("Helvetica");
    BodyLined = true;
	BodyTreeLine = true;
	DisplayPageNumber = false;
		
	//---------------------------------------------------
	// enable/disable some menu entries
	//---------------------------------------------------
	EnableComponents();
	
	//-------------------------------------------------------------------------
	// enable mouse tracking to adjust mouse position field in the status bar
	//-------------------------------------------------------------------------
	setMouseTracking(true);	
	
	//---------------------------------------------------
	// init qimageio extensions
	//---------------------------------------------------
	qInitImageIO();
  	
}

KCDLabel::~KCDLabel()
{
	delete myDirTree;
	delete BodyFont;
}

/* user press a key in the widget --> intercept page down/up and the like */
void KCDLabel::keyPressEvent ( QKeyEvent * e )
{
	switch ( e->key() )
		{
		case Key_Down :
		case Key_PageDown :
			ObjectSelected = NULL;
			CurrentPageToDisplay++;
			if (CurrentPageToDisplay==MAX_PAGES+1) CurrentPageToDisplay=MAX_PAGES;
			AdjustStatus();
			break;
		case Key_Up :
		case Key_PageUp :
			ObjectSelected = NULL;
			CurrentPageToDisplay--;		
			if (CurrentPageToDisplay==-1) CurrentPageToDisplay=0;
			AdjustStatus();
			break;
		case Key_Home :
			ObjectSelected = NULL;
			CurrentPageToDisplay = 0;
			AdjustStatus();
			break;
		case Key_End :
			ObjectSelected = NULL;
			CurrentPageToDisplay = MAX_PAGES;
			AdjustStatus();
			break;
		case Key_Delete :
			slotObjectDelete();
			break;
		case Key_Plus :
			if (ObjectSelected)
				if (ObjectSelected->GetClassType()==TYPE_CLASS_IMAGE)
					slotImageIncreaseOrder();
				else
					slotStringIncreaseOrder();
			break;
		case Key_Minus :
			if (ObjectSelected)
				if (ObjectSelected->GetClassType()==TYPE_CLASS_IMAGE)
					slotImageDecreaseOrder();
				else
					slotStringDecreaseOrder();
			break;
		}
	repaint();
}


/** Open a saved cd-label file */
void KCDLabel::slotFileOpen()
{
}

/** Save a cd-label into a file */
void KCDLabel::slotFileSave()
{
}

/** Read the CD Content  */
void KCDLabel::slotBodyReadCD()
{
	myDirTree->Clear();	
	myDirTree->SetBaseDir(SourceCD);
	myDirTree->Traverse();
	slotNumberLevel(Levels);
}

/** Edit the text for the side of the CD */
void KCDLabel::slotSlideEditText()
{
	// open the slide edit dialog box
	SideEditDlg * dlg = new SideEditDlg(this, i18n("Slide Text"), &BackPage);
	dlg->exec();
}

/** go to the front page */
void KCDLabel::slotPageFront()
{
	CurrentPageToDisplay = 1;
	repaint();
}

/** go to back page */
void KCDLabel::slotPageBack()
{
	CurrentPageToDisplay = 0;
	repaint();
}

/** go to page number */
void KCDLabel::slotPageNumber()
{
	PageNumberDlg * dlg = new PageNumberDlg(this, i18n("Goto page..."), &CurrentPageToDisplay);
	dlg->exec();
	repaint();
}

/** Print the cd label */
void KCDLabel::slotFilePrint()
{
	if (Printer->setup())
		{
		int SavedTopX = TopX;
		int SavedTopY = TopY;
		TopX = 50;			// set top left point for drawings on the printer
		TopY = 50;
		Printing = true;
		QPainter paint;
		paint.begin( Printer );
		PaintBackPage(&paint);
		paint.translate( 0, BACK_V+1 );		// before it was (0, BACK_V+10)
		PaintFrontPage(&paint, 0);
		Printing = false;
		int curpage = 1;
		paint.translate(0,-BACK_V-1);			// before it was (0, -BACK_V-10)
		while ( FrontPages[curpage].NumberOfObjects || FrontPages[curpage].PageToPrint )
			{
			Printing = true;
			Printer->newPage();
			PaintFrontPage(&paint, curpage++ );
			paint.translate( 0, FRONT_V+1); 	// before it was (0, FRONT_V+10)
			PaintFrontPage(&paint, curpage++ );
			paint.translate( 0, -FRONT_V-1);	// before it was (0, -FRONT_V-10)		
			}
		Printer->newPage();
		paint.end();
		TopX = SavedTopX;
		TopY = SavedTopY;
		}
}

/** call the print setup dialog */
void KCDLabel::slotFilePrintSetup()
{
}

/** create a new cd label */
void KCDLabel::slotFileNew()
{
	// erase all pages content
	BackPage.Clear();
	for (int i=0;i<MAX_PAGES;i++)
		FrontPages[i].Clear();
	slotNumberColumn(COLUMN_0);
}

/** Draw the back page of the cd cover */
void KCDLabel::PaintBackPage(QPainter * paint)
{
	// Erase previous drawing and draw the layout
    bufferpixmap.fill(white);

    paint->setPen( black );
    paint->drawLine( TopX, TopY, TopX+BACK_HI+BACK_HS*2, TopY );
    paint->drawLine( TopX, TopY+BACK_V, TopX+BACK_HI+BACK_HS*2, TopY+BACK_V );
    paint->drawLine( TopX, TopY, TopX, TopY+BACK_V );
    paint->drawLine( TopX+BACK_HI+BACK_HS*2, TopY, TopX+BACK_HI+BACK_HS*2, TopY+BACK_V );
	paint->drawLine( TopX+BACK_HS, TopY, TopX+BACK_HS, TopY+BACK_V );
    paint->drawLine( TopX+BACK_HS+BACK_HI, TopY, TopX+BACK_HS+BACK_HI, TopY+BACK_V );

    // draw the images
    if (BackPage.Images.size())
    	{
    	for (unsigned int i=0;i<BackPage.Images.size();i++)
    		{
    		KCoverImage * img = BackPage.GetImage(i);
    		QPoint pos(TopX+img->X, TopY+img->Y);
    		paint->drawImage( pos, img->Image );
    		}
    	}

    // draw the texts
    if (BackPage.Strings.size())
    	{
    	for (unsigned int i=0;i<BackPage.Strings.size();i++)
    		{
    		KCoverString * str = BackPage.GetString(i);
    		QPoint pos( TopX+str->X, TopY+str->Y );
    		paint->setFont(*(str->Font->Font));
    		paint->setPen(*(str->textcolor));
    		paint->setBackgroundMode(str->transparentbackground ? TransparentMode : OpaqueMode );
    		paint->setBackgroundColor(*(str->backcolor));    		
    		paint->drawText( pos, str->Text.c_str() );
    		}
    	}

    	
    // display the side texts
    if (!SlideTransparentBackground)
    	{
    	paint->fillRect( TopX+1, TopY+1, BACK_HS-1, BACK_V-1, QBrush(*(BackPage.SideA->Font->back)));
    	paint->fillRect( TopX+BACK_HS+BACK_HI+1, TopY+1, BACK_HS-1, BACK_V-1, QBrush(*(BackPage.SideB->Font->back)));
    	}
    // Side A
    paint->setFont(*(BackPage.SideA->Font->Font));
    paint->translate( TopX, TopY+BACK_V );
    paint->rotate( -90 );
    paint->setPen(*(BackPage.SideA->Font->front));
    paint->drawText( 10, 0, BACK_V-10, BACK_HS, Slide_Align | AlignVCenter, BackPage.SideA->Text.c_str(), BackPage.SideA->Text.length() );
	paint->rotate( 90 );
	paint->translate( -TopX, -TopY-BACK_V );
	
    // Side B
    paint->setPen(*(BackPage.SideB->Font->front));
    paint->setFont(*(BackPage.SideB->Font->Font));
    //paint->translate( BACK_V, BACK_HS+BACK_HI );
    paint->translate( TopX+BACK_HI+2*BACK_HS, TopY );
    paint->rotate( 90 );
    paint->drawText( 10, BACK_HS-1, BACK_V-10, -BACK_HS, Slide_Align | AlignVCenter, BackPage.SideB->Text.c_str(), BackPage.SideB->Text.length() );

    // transform again to set all the other things normally !
    paint->rotate( -90 );
    //paint->translate( 0, BACK_HI );
    paint->translate( -TopX-BACK_HI-2*BACK_HS, -TopY );

    // draw the body content
   	QFontMetrics FontMetric(*BodyFont);
   	int LineHeight = FontMetric.height()+3;	// must be adapted according the size of the font
    if ( Columns && BodyPage==0 )
    	{
    	paint->setPen( black );
    	int ColumnWidth = BACK_HI / Columns - 8 ;  	// columns width in pixels
    	// draw the vertical colums lines
    	for (int i=0;i<Columns-1;i++)
    		paint->drawLine( TopX + (i+1)*(ColumnWidth+8)+BACK_HS, TopY+4, BACK_HS+TopX + (i+1)*(ColumnWidth+8), TopY+BACK_V-4);
    	// draw the text content of the columns
    	int MaxLines = (BACK_V/*-LineHeight*/) / LineHeight;
    	//int MaxItemsPerPage = Columns * MaxLines ;
    	int Item = 0;
    	paint->setFont(*BodyFont);
    	for (int Col=0; ( Col < Columns ) && ( Item < myDirTree->GetNumberOfTreeEntries() ); Col++ )
    		{
    		BackPage.PageToPrint = true;
    		for (int Line=0; ( Line<MaxLines ) && ( Item < myDirTree->GetNumberOfTreeEntries() ); Line++)
    			{
    			if (myDirTree->GetResultTreeEntry(Item).Type == TYPE_DIR)
    				BodyFont->setBold(true);
    			else
    				BodyFont->setBold(false);    			
    			// draw a horizontal line if needed
    			if ( BodyLined )
    				paint->drawLine( BACK_HS+4+TopX+(ColumnWidth+8)*Col, TopY+((Line+1)*LineHeight), BACK_HS+4+TopX+(ColumnWidth+8)*Col+ColumnWidth, TopY+((Line+1)*LineHeight) );
    			// draw the tree structure line if needed
    			int currentlevel = myDirTree->GetResultTreeEntry(Item).Level;
    			if ( BodyTreeLine && currentlevel > 1 )
    				{
    				int Left_X 		= 6 + TopX + (ColumnWidth+8) * Col + BACK_HS;
    				int Top_Y 		= TopY+((Line)*LineHeight) - 3 ;
    				if ( Top_Y<TopY ) Top_Y = Top_Y + 6;
    				int Bottom_Y	= Top_Y + LineHeight;
    				int n;
    				for (n=0; n < myDirTree->GetResultTreeEntry(Item).Level-2; n++ )
    					{
    					paint->drawLine( Left_X + n*7, Top_Y, Left_X + n*7, Bottom_Y );
    					}
    				if ( n>=0 )
    					{
    					if ( Item<myDirTree->GetNumberOfTreeEntries() && Item<myDirTree->GetNumberOfTreeEntries()-1?(myDirTree->GetResultTreeEntry(Item+1).Level >= myDirTree->GetResultTreeEntry(Item).Level):false )
	    					paint->drawLine( Left_X + n*7, Top_Y, Left_X + n*7, Bottom_Y );
	    				else
	    					paint->drawLine( Left_X + n*7, Top_Y, Left_X + n*7, Top_Y + ((Bottom_Y - Top_Y+3)/2) );
    					paint->drawLine( Left_X + n*7, Top_Y + ((Bottom_Y - Top_Y + 3)/2), Left_X + n*7 + 2, Top_Y + ((Bottom_Y - Top_Y+3)/2) );
    					}
	  				}
    			
    			paint->setFont(*BodyFont);
    			paint->drawText( BACK_HS+4 + TopX + (ColumnWidth+8) * Col + ( myDirTree->GetResultTreeEntry(Item).Level-1 ) * 7,
    							 TopY + Line * LineHeight + 2,
    							 ColumnWidth - ( myDirTree->GetResultTreeEntry(Item).Level-1 ) * 7,
    							 LineHeight,
    							 AlignTop | SingleLine | ShowPrefix /*| WordBreak*/,
    							 myDirTree->GetResultTreeEntry(Item++).FileName.c_str() );
    			}
    		}
    	}
}

/** Draw the back page of the cd cover */
void KCDLabel::PaintFrontPage(QPainter * paint, int page)
{
	// make page not to print now
	FrontPages[page].PageToPrint = false;
	
	// Erase previous drawing and draw the layout
    bufferpixmap.fill(white);

    paint->setPen( black );

    paint->drawLine( TopX, TopY, TopX+FRONT_H, TopY );
    paint->drawLine( TopX, TopY+FRONT_V, TopX+FRONT_H, TopY+FRONT_V );
    paint->drawLine( TopX, TopY, TopX, TopY+FRONT_V );
    paint->drawLine( TopX+FRONT_H, TopY, TopX+FRONT_H, TopY+FRONT_V );
	
    // draw the images
    // --- THIS PART MUST BE DONE CORRECTLY !!! --- //
    if (FrontPages[page].Images.size())
    	{
    	for (unsigned int i=0;i<FrontPages[page].Images.size();i++)
    		{
    		KCoverImage * img = FrontPages[page].GetImage(i);
    		QPoint pos(TopX+img->X, TopY+img->Y);
    		paint->drawImage( pos, img->Image );
    		}
    	}

    // draw the texts
    if (FrontPages[page].Strings.size())
    	{
    	for (unsigned int i=0;i<FrontPages[page].Strings.size();i++)
    		{
    		KCoverString * str = FrontPages[page].GetString(i);
    		QPoint pos( TopX+str->X, TopY+str->Y );
    		paint->setFont(*(str->Font->Font));
    		paint->setPen(*(str->textcolor));
    		paint->setBackgroundMode(str->transparentbackground ? TransparentMode : OpaqueMode );
    		paint->setBackgroundColor(*(str->backcolor));    		
    		paint->drawText( pos, str->Text.c_str() );
    		}
    	}
    	
    // draw the body content
   	QFontMetrics FontMetric(*BodyFont);
   	int LineHeight = FontMetric.height()+3;	// must be adapted according the size of the font
    if ( Columns && page+1>=BodyPage && BodyPage )
    	{
    	paint->setPen( black );
    	int ColumnWidth = FRONT_H / Columns - 8 ;  	// columns width in pixels
    	// draw the vertical colums lines
    	for (int i=0;i<Columns-1;i++)
    		paint->drawLine( TopX + (i+1)*(ColumnWidth+8), TopY+4, TopX + (i+1)*(ColumnWidth+8), TopY+FRONT_V-4);
    	// draw the text content of the columns
    	int MaxLines = (FRONT_V/*-LineHeight*/) / LineHeight;
    	int MaxItemsPerPage = Columns * MaxLines ;
    	int Item = ( 1+page - BodyPage ) * MaxItemsPerPage;
    	paint->setFont(*BodyFont);
    	for (int Col=0; ( Col < Columns ) && ( Item < myDirTree->GetNumberOfTreeEntries() ); Col++ )
    		{
    		FrontPages[page].PageToPrint = true;
    		for (int Line=0; ( Line<MaxLines ) && ( Item < myDirTree->GetNumberOfTreeEntries() ); Line++)
    			{
    			if (myDirTree->GetResultTreeEntry(Item).Type == TYPE_DIR)
    				BodyFont->setBold(true);
    			else
    				BodyFont->setBold(false);    			
    			// draw a horizontal line if needed
    			if ( BodyLined )
    				paint->drawLine( 4+TopX+(ColumnWidth+8)*Col, TopY+((Line+1)*LineHeight), 4+TopX+(ColumnWidth+8)*Col+ColumnWidth, TopY+((Line+1)*LineHeight) );
    			// draw the tree structure line if needed
    			int currentlevel = myDirTree->GetResultTreeEntry(Item).Level;
    			if ( BodyTreeLine && currentlevel > 1 )
    				{
    				int Left_X 		= 6 + TopX + (ColumnWidth+8) * Col ;
    				int Top_Y 		= TopY+((Line)*LineHeight) - 3 ;
    				if ( Top_Y<TopY ) Top_Y = Top_Y + 6;
    				int Bottom_Y	= Top_Y + LineHeight;
    				int n;
    				for (n=0; n < myDirTree->GetResultTreeEntry(Item).Level-2; n++ )
    					{
    					paint->drawLine( Left_X + n*7, Top_Y, Left_X + n*7, Bottom_Y );
    					}
    				if ( n>=0 )
    					{
    					if ( Item<myDirTree->GetNumberOfTreeEntries() && Item<myDirTree->GetNumberOfTreeEntries()-1?(myDirTree->GetResultTreeEntry(Item+1).Level >= myDirTree->GetResultTreeEntry(Item).Level):false )
	    					paint->drawLine( Left_X + n*7, Top_Y, Left_X + n*7, Bottom_Y );
	    				else
	    					paint->drawLine( Left_X + n*7, Top_Y, Left_X + n*7, Top_Y + ((Bottom_Y - Top_Y+3)/2) );
    					paint->drawLine( Left_X + n*7, Top_Y + ((Bottom_Y - Top_Y + 3)/2), Left_X + n*7 + 2, Top_Y + ((Bottom_Y - Top_Y+3)/2) );
    					}
	  				}
    			
    			paint->setFont(*BodyFont);
    			paint->drawText( 4 + TopX + (ColumnWidth+8) * Col + ( myDirTree->GetResultTreeEntry(Item).Level-1 ) * 7,
    							 TopY + Line * LineHeight + 2,
    							 ColumnWidth - ( myDirTree->GetResultTreeEntry(Item).Level-1 ) * 7,
    							 LineHeight,
    							 AlignTop | SingleLine | ShowPrefix /*| WordBreak*/,
    							 myDirTree->GetResultTreeEntry(Item++).FileName.c_str() );
    			}
    		}
    	}
    // display page number
    if (DisplayPageNumber && page+1>=BodyPage )
    	{
    	char tmp[10];
    	sprintf(tmp,"%d", page+1);
    	int size = BodyFont->pointSize();
    	BodyFont->setPointSize(12);
		BodyFont->setBold(false);
		paint->setFont(*BodyFont);
		paint->drawText( TopX + FRONT_H - 20,
						 TopY + FRONT_V - 15,
						 50,
						 LineHeight,
						 AlignTop | SingleLine | ShowPrefix /*| WordBreak*/,
						 tmp );
		BodyFont->setPointSize(size);
    	}
    	
}

/** define the paint event */
void KCDLabel::paintEvent( QPaintEvent * event)
{
	QPainter *paint = new QPainter(&bufferpixmap);
	switch ( CurrentPageToDisplay )
		{
		case 0 :
			TopX = ( width() - (BACK_HI+BACK_HS*2) ) / 2;
			TopY = menubar->height() + ( height() - menubar->height() - statusbar->height() - BACK_V ) / 2;
			PaintBackPage(paint);
			break;
		default :
			TopX = ( width() - FRONT_H ) / 2;
			TopY = menubar->height() + ( height() - menubar->height() - statusbar->height() - FRONT_V ) / 2;
			PaintFrontPage(paint, CurrentPageToDisplay-1);
			break;
		}
	
    if ( ObjectSelected!=NULL )
    	{
    	paint->setPen( red );
    	if (ObjectSelected->GetClassType()==TYPE_CLASS_IMAGE)
	    	paint->drawRect(TopX+ObjectSelected->X-5, TopY+ObjectSelected->Y-5, ObjectSelected->Width()+10, ObjectSelected->Height()+10);
	    else
			{
			KCoverString * str = dynamic_cast<KCoverString *>(ObjectSelected);
			paint->drawRect(TopX+str->RealX()-10, TopY+str->RealY()-10, str->Width()+20, str->Height()+20);
			}
    	}
	bitBlt(this, event->rect().topLeft(), &bufferpixmap, event->rect());
	DisplayCurrentPage();
	delete paint;
}

/** display the current page in the status bar */
void KCDLabel::DisplayCurrentPage() const
{
	char buf[20];
	sprintf(buf,i18n("Page %d"), CurrentPageToDisplay);
	statusbar->changeItem( buf, STATUS_PAGE );
}

/** Draw a rotated text (taken from the QT book) */
void KCDLabel::DrawRotatedText(QPainter * painter, float degrees, int x, int y, char * txt) const
{
	painter->rotate( degrees );
	QPoint pos( x, y );
	pos = painter->xFormDev( pos );
	painter->drawText( pos, txt );
	painter->rotate( -degrees );
}

/** Select the side text alignement property */
void KCDLabel::slotAlignMenu(int id)
{
    alignmenu->setItemChecked(AlignChecked, false);	
    alignmenu->setItemChecked(id, true);	
	switch (id)
		{
		case ALIGN_LEFT :
			Slide_Align = AlignLeft;
			break;
		case ALIGN_CENTER :
			Slide_Align = AlignHCenter;
			break;
		case ALIGN_RIGHT :
			Slide_Align = AlignRight;
			break;
		}
	AlignChecked = id;
}

/** insert an image to the current page */
void KCDLabel::slotImageInsert()
{
    ImageFilename = QFileDialog::getOpenFileName(0, "*", this);
    if (ImageFilename.isEmpty())
    	return;
    	
    // add the image to the images list of the current displayed page
    if (CurrentPageToDisplay==0)	// this is the back page
		BackPage.AddImage( KCoverImage((const char *)(ImageFilename),0,0,0) );
    else
    	FrontPages[CurrentPageToDisplay-1].AddImage( KCoverImage((const char *)(ImageFilename),0,0,0) );
    repaint();
}

/** Resize an image to the full page size */
void KCDLabel::slotImageResizeFullPage()
{
	// if no object selected or object is not an image, return !
	if ( ObjectSelected==NULL || (ObjectSelected->GetClassType()!=TYPE_CLASS_IMAGE) ) return;
	
	// we are on backpage
	if (CurrentPageToDisplay==0)
		{
		ImageSelected = dynamic_cast<KCoverImage *>(ObjectSelected);
		ImageSelected->Image = ImageSelected->Image.smoothScale( BACK_HI+BACK_HS*2, BACK_V );
		}
	else	// we are on a front page
		{
		ImageSelected = dynamic_cast<KCoverImage *>(ObjectSelected);
		ImageSelected->Image = ImageSelected->Image.smoothScale( FRONT_H, FRONT_V );
		}
	repaint();
}

void KCDLabel::mouseDoubleClickEvent(QMouseEvent * event)
{
	// position of the drawn top left corner of the page	
	QPoint clickedpos = event->pos() - QPoint( TopX, TopY );

    if ( ObjectSelected!=NULL )
    	{
    	if (ObjectSelected->IsPointInObject(clickedpos))
    		{
   			ObjectSelected = NULL;
   			repaint();
   			EnableComponents();
   			statusbar->changeItem( "", STATUS_MSG);	// clear the status bar message
   			return;
    		}
    	}
    		
	if (CurrentPageToDisplay==0)
		ObjectSelected = BackPage.GetSelectedObject(clickedpos);
	else
		ObjectSelected = FrontPages[CurrentPageToDisplay-1].GetSelectedObject(clickedpos);	

	// set the status bar message
	AdjustStatus();
	repaint();
	EnableComponents();
}

void KCDLabel::mouseMoveEvent( QMouseEvent * event)
{
    QPoint pos = event->pos() - QPoint( TopX, TopY );

    // update the mouse pos status
    char buf[50];
    sprintf(buf,"(%d, %d)", pos.x(), pos.y());
	statusbar->changeItem(buf, STATUS_MOUSEPOS);
    	
	// if leftbutton not pressed --> return
	if ( event->state() != LeftButton )
		return;
			
    if ( InitialPosForMove==QPoint(0,0) )
    	{
    	InitialPosForMove = pos;
    	return;
    	}
    	
    if ( ObjectSelected!=NULL )
    	{
    	//if (!ObjectSelected->IsPointInObject(pos))
    	//	return;
   		ObjectSelected->X += pos.x()-InitialPosForMove.x();
   		ObjectSelected->Y += pos.y()-InitialPosForMove.y();
   		InitialPosForMove = pos;
		AdjustStatus();
   		repaint();
    	}
}

/** mouse press event */
void KCDLabel::mousePressEvent( QMouseEvent * event )
{
	if (event->button()==RightButton )
		{
		ClickedPos = event->pos() - QPoint(TopX, TopY);
		if (ObjectSelected!=NULL)
			if ( ObjectSelected->GetClassType()==TYPE_CLASS_IMAGE )
				ImagePopupMenu->exec(QCursor::pos());
			else
				StringPopupMenu->exec(QCursor::pos());
		else
			GlobalPopupMenu->exec(QCursor::pos());
		}
}

/** mouse button released */
void KCDLabel::mouseReleaseEvent ( QMouseEvent * )
{
	InitialPosForMove = QPoint(0,0);
}

/** manage the resizing of the window */
void KCDLabel::resizeEvent(QResizeEvent * event)
{
	bufferpixmap.resize(event->size());
	KTopLevelWidget::resizeEvent(event);
}
/** make the slide background transparent */
void KCDLabel::slotImageTransparent(int id)
{
	if (id==SLIDE_BACKGROUND)
		{
		SlideTransparentBackground = !SlideTransparentBackground;
		slidemenu->setItemChecked(SLIDE_BACKGROUND, SlideTransparentBackground);
		}
}

/** align the image to the center of the page */
void KCDLabel::slotImageAlignCenter()
{
    if ( ObjectSelected!=NULL )
    	if ( ObjectSelected->GetClassType()==TYPE_CLASS_IMAGE )
    		{
    		ImageSelected = dynamic_cast<KCoverImage *>(ObjectSelected);
    		if (CurrentPageToDisplay == 0)
    			{
    			ImageSelected->X = ((BACK_HI+2*BACK_HS)/2) - (ImageSelected->Width()/2);
    			ImageSelected->Y = (BACK_V/2) - (ImageSelected->Height()/2);
    			}
    		else
    			{
    			ImageSelected->X = (FRONT_H/2) - (ImageSelected->Width()/2);
    			ImageSelected->Y = (FRONT_V/2) - (ImageSelected->Height()/2);
    			}
    		}
    repaint();
}

/** resize image to xxx % or pixels */
void KCDLabel::slotImageResize()
{
	if (ObjectSelected==NULL)
		return;
	if ( ObjectSelected->GetClassType()!=TYPE_CLASS_IMAGE )
		return;
	ScaleImageDlg * dlg = new ScaleImageDlg(this, i18n("Scale"), dynamic_cast<KCoverImage *>(ObjectSelected));
	
	dlg->exec();
}

/**  */
void KCDLabel::slotImageIncreaseOrder()
{
	if (!ObjectSelected) return;
	ObjectSelected->Order++;
	if (CurrentPageToDisplay==0)
		BackPage.SortImages();
	else
		FrontPages[CurrentPageToDisplay-1].SortImages();
	AdjustStatus();
}

/**  */
void KCDLabel::slotImageDecreaseOrder()
{
	if (!ObjectSelected) return;
	ObjectSelected->Order--;
	if (CurrentPageToDisplay==0)
		BackPage.SortImages();
	else
		FrontPages[CurrentPageToDisplay-1].SortImages();
	AdjustStatus();
}
/** display the about box */
void KCDLabel::slotHelpAbout()
{
	About * myabout = new About(this, i18n("About..."));
	myabout->exec();
}

/** insert a string on a page */
void KCDLabel::slotStringInsert()
{
	StringEditDlg * dlg;
	
	// open the slide edit dialog box
	if (CurrentPageToDisplay==0)
		dlg = new StringEditDlg(this, i18n("Insert a String"), &BackPage, NULL, ClickedPos);
	else
		dlg = new StringEditDlg(this, i18n("Insert a String"), &FrontPages[CurrentPageToDisplay-1], NULL, ClickedPos);
		
	dlg->exec();
}

/** delete selected string */
void KCDLabel::slotObjectDelete()
{
	if (ObjectSelected)
		{
		if (CurrentPageToDisplay==0)
			BackPage.DeleteSelectedObject(ObjectSelected);
		else
			FrontPages[CurrentPageToDisplay-1].DeleteSelectedObject(ObjectSelected);
		ObjectSelected = NULL;
		}
	EnableComponents();
	AdjustStatus();
}

/** modify the selected string */
void KCDLabel::slotStringModify()
{
	if (ObjectSelected==NULL)
		return;
	if (ObjectSelected->GetClassType()!=TYPE_CLASS_STRING)
		return;
	
	StringEditDlg * dlg;
	
	// open the slide edit dialog box
	if (CurrentPageToDisplay==0)
		dlg = new StringEditDlg(this, i18n("Insert a String"), NULL, dynamic_cast<KCoverString *>(ObjectSelected), ClickedPos);
	else
		dlg = new StringEditDlg(this, i18n("Insert a String"), NULL, dynamic_cast<KCoverString *>(ObjectSelected), ClickedPos);
		
	dlg->exec();
		
}

/**  */
void KCDLabel::slotStringIncreaseOrder()
{
	if (!ObjectSelected) return;
	ObjectSelected->Order++;
	if (CurrentPageToDisplay==0)
		BackPage.SortStrings();
	else
		FrontPages[CurrentPageToDisplay-1].SortStrings();
	AdjustStatus();
}

/**  */
void KCDLabel::slotStringDecreaseOrder()
{
	if (!ObjectSelected) return;
	ObjectSelected->Order--;
	if (CurrentPageToDisplay==0)
		BackPage.SortStrings();
	else
		FrontPages[CurrentPageToDisplay-1].SortStrings();
	AdjustStatus();
}

/** Enable/disable some controls according to the selection */
void KCDLabel::EnableComponents()
{
	// Disable/enable string entries
	stringmenu->setItemEnabled( STRING_MENU_DELETE, ObjectSelected && ObjectSelected->GetClassType()==TYPE_CLASS_STRING );
	stringmenu->setItemEnabled( STRING_MENU_MODIFY, ObjectSelected && ObjectSelected->GetClassType()==TYPE_CLASS_STRING );
	stringmenu->setItemEnabled( STRING_MENU_ORDER, ObjectSelected && ObjectSelected->GetClassType()==TYPE_CLASS_STRING );
	
	// Disable/enable images entries
	imagemenu->setItemEnabled( IMAGE_MENU_DELETE, ObjectSelected && ObjectSelected->GetClassType()==TYPE_CLASS_IMAGE );
	imagemenu->setItemEnabled( IMAGE_MENU_RESIZEFULL, ObjectSelected && ObjectSelected->GetClassType()==TYPE_CLASS_IMAGE );
	imagemenu->setItemEnabled( IMAGE_MENU_RESIZE, ObjectSelected && ObjectSelected->GetClassType()==TYPE_CLASS_IMAGE );
	imagemenu->setItemEnabled( IMAGE_MENU_ALIGN, ObjectSelected && ObjectSelected->GetClassType()==TYPE_CLASS_IMAGE );
	imagemenu->setItemEnabled( IMAGE_MENU_ORDER, ObjectSelected && ObjectSelected->GetClassType()==TYPE_CLASS_IMAGE );	
}

/** select the source dircetory to read from */
void KCDLabel::slotBodySelectSource()
{
	SourceSelectDlg * dlg = new SourceSelectDlg(this, "Source Directory",  SourceCD);
	dlg->exec();
}

/** set the number of columns to display */
void KCDLabel::slotNumberColumn(int id)
{
	columnmenu->setItemChecked(Columns, false);
	Columns = id - COLUMN_0;
	columnmenu->setItemChecked(Columns, true);	
}

/** fix the number of levels to scan */
void KCDLabel::slotNumberLevel(int id)
{
	levelmenu->setItemChecked(Levels, false);
	Levels = id;
	levelmenu->setItemChecked(Levels, true);
	int levels = id - LEVEL_ALL - 1;
	if (id==LEVEL_ALL) levels=999999;
	myDirTree->BuildResultTree( levels );
}

/** select font to use for body */
void KCDLabel::slotBodySelectFont()
{
	KFontDialog *fontdlg = new KFontDialog(this, i18n("Select Font"), true);
	QObject::connect(fontdlg, SIGNAL(fontSelected(const QFont &)), this, SLOT(slotBodyFontSelected(const QFont &)));
	fontdlg->setFont(*BodyFont);
	fontdlg->exec();
}

/** user select a font */
void KCDLabel::slotBodyFontSelected(const QFont &font)
{
	if (BodyFont)
		delete BodyFont;
	BodyFont = new QFont(font);
}
/** user select line or not */
void KCDLabel::slotBodyLined(int id)
{
	if (id==BODY_LINE)
		{
		BodyLined = !BodyLined;
		bodymenu->setItemChecked(BODY_LINE, BodyLined );
		}
	
}
/** user select to draw or not the tree lines */
void KCDLabel::slotBodyDrawTree(int id)
{
	if (id==TREE_LINE)
		{
		BodyTreeLine = !BodyTreeLine;
		bodymenu->setItemChecked( TREE_LINE, BodyTreeLine );
		}
}
/** enable page number display */
void KCDLabel::slotBodyPageNumber(int id)
{
	if (id==DISPLAY_PAGENUMBER)
		{
		DisplayPageNumber = !DisplayPageNumber;
		bodymenu->setItemChecked( DISPLAY_PAGENUMBER, DisplayPageNumber);
		}
}

/** display body layout dialog */
void KCDLabel::slotBodyLayout()
{
	BodyLayout * dlg = new BodyLayout(this, "Choose body layout", &BodyPage);
	dlg->exec();
}

/** invoke the help manual */
void KCDLabel::slotHelpManual()
{
	kapp->invokeHTMLHelp("kcdlabel/index.html", "");
}
/** display the correct status info according to selected object */
void KCDLabel::AdjustStatus()
{
	char buf[100];
	
	if ( ObjectSelected )
		sprintf(buf, i18n("Select.Obj. : Pos=(%d,%d), Order=%d"), ObjectSelected->X, ObjectSelected->Y, ObjectSelected->Order );
	else
		buf[0]=0;		
	
	statusbar->changeItem(buf, STATUS_MSG);
	
}
