/***************************************************************************
                          kcoverobject.h  -  description
                             -------------------
    begin                : Mon Apr 3 2000
    copyright            : (C) 2000 by Pascal 'PeP' Panneels
    email                : pp@bvdep.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KCOVEROBJECT_H
#define KCOVEROBJECT_H

#include <qpoint.h>

/**Base object for anything that could be drawn on the page layout; some classes should 
inherit from this (KCoverImage and KCoverString)

This is a Abstract class; you cannot instantiate object of this class; you have to overload
the function GetClassType, which should return 'TYPE_CLASS_xxx'

  *@author Pascal 'PeP' Panneels
  */

#define TYPE_CLASS_IMAGE		1
#define TYPE_CLASS_STRING		2

#define ALIGN_LEFT			1
#define ALIGN_CENTER		2
#define ALIGN_RIGHT			3

class KCoverObject
{
public: 
  KCoverObject(int x=0, int y=0, int order=0)
		{ X=x; Y=y; Order=order; }
  virtual ~KCoverObject() {}
  /** return the class type */
  virtual int GetClassType()=0 ;	// pure virtual
  /** return true is a point is in a object or not */
  virtual bool IsPointInObject(QPoint &pos)=0;
  /**  return the widht of the gfx object */
  virtual int Width()=0;
  /**  return the height of the gfx object */
  virtual int Height()=0;

  /** used for sorting on "Order" field */
  bool operator < (KCoverObject &p);

	
public :
	// relative position to 0,0 = upper left corner
	int X, Y;
	// relative order to the other object (0=most behind=first to be drawn)
	int Order;
};

#endif
