/***************************************************************************
                          kcoverpage.cpp  -  description
                             -------------------
    begin                : Sat Apr 15 2000
    copyright            : (C) 2000 by Pascal 'PeP' Panneels
    email                : pp@bvdep.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kcoverpage.h"

KCoverPage::KCoverPage()
{
	NumberOfObjects=0;
	PageToPrint = false;
}

KCoverPage::~KCoverPage()
{
}

KCoverObject * KCoverPage::GetSelectedObject(QPoint &p)
{
	// look in the strings list, scanning from top to beyond
	// ... must be done !
	for (std::list<KCoverString>::reverse_iterator i=Strings.rbegin(); i!=Strings.rend();i++)
		if ( (*i).IsPointInObject(p) )
			return &(*i);
	
	// look in the images list, scanning from top to beyond
	// !!! must be changed !!!
	for (std::list<KCoverImage>::reverse_iterator i=Images.rbegin(); i!=Images.rend();i++)
		//if ( (p.x()>=(*i).X) && (p.x()<=(*i).X+(*i).Image.width()) && (p.y()>=(*i).Y) && (p.y()<=(*i).Y+(*i).Image.height()) )
		if ( (*i).IsPointInObject(p) )
			return &(*i);
		
	return NULL;
}

/** add an image to the list */
void KCoverPage::AddImage(KCoverImage img)
{
	bool cont = true;
	std::list<KCoverImage>::iterator j=Images.begin();
	
    while ( (img.Order > (*j).Order) && cont && ( j!=Images.end()) )
    	j++;
	Images.insert(j, img);
	NumberOfObjects++;
}

/** return the image #i of the list */
KCoverImage * KCoverPage::GetImage(int i)
{
	std::list<KCoverImage>::iterator j=Images.begin();
	for (; i>0; i--)
		j++;
	return &(*j);
}

/** Erase the selected object */
void KCoverPage::DeleteSelectedObject(KCoverObject *obj)
{
	if (obj->GetClassType()==TYPE_CLASS_IMAGE)
		{
		KCoverImage * img = dynamic_cast<KCoverImage *>(obj);
		std::list<KCoverImage>::iterator j;
		j = std::find(Images.begin(), Images.end(), *img );
		Images.erase(j);
		NumberOfObjects--;
		}
	else
		{
		KCoverString * str = dynamic_cast<KCoverString *>(obj);
		std::list<KCoverString>::iterator j;
		j = std::find(Strings.begin(), Strings.end(), *str );
		Strings.erase(j);
		NumberOfObjects--;
		}
}

/**  sort the image list */
void KCoverPage::SortImages()
{
	Images.sort();
}

/** add a string to the page */
void KCoverPage::AddString(KCoverString str)
{
	bool cont = true;
	std::list<KCoverString>::iterator j=Strings.begin();
	
    while ( (str.Order > (*j).Order) && cont && ( j!=Strings.end()) )
    	j++;
	Strings.insert(j, str);
	NumberOfObjects++;
}

/** sort the string list */
void KCoverPage::SortStrings()
{
	Strings.sort();
}

/** return the string number i */
KCoverString * KCoverPage::GetString(int i)
{
	std::list<KCoverString>::iterator j=Strings.begin();
	for (; i>0; i--)
		j++;
	return &(*j);
}

/** clear the page content */
void KCoverPage::Clear()
{
	for (unsigned int i=0;i<Images.size();i++)
		Images.pop_back();
	for (unsigned int i=0;i<Strings.size();i++)
		Strings.pop_back();
	NumberOfObjects = 0;
	PageToPrint = false;
}
