/***************************************************************************
                          kdirtree.cpp  -  description
                             -------------------
    begin                : Sun Apr 30 2000
    copyright            : (C) 2000 by Pascal 'PeP' Panneels
    email                : pp@bvdep.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kdirtree.h"
#include <dirent.h>
#include <string>
#include <vector.h>
#include <algo.h>
#include <error.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/stat.h>
#include <time.h>

KDirTree::KDirTree( const char * basedir )
{
	BaseDir = string( basedir );
	NumberOfFiles = 0;
	NumberOfDirectories = 0;
}

KDirTree::~KDirTree()
{
}

void KDirTree::SetBaseDir( const char * basedir )
{
	BaseDir = string( basedir );
}

void KDirTree::Traverse()
{
	DIR * DirFD;
	// try to open the directory
	if ( (DirFD = opendir( BaseDir.c_str())) == NULL )	// open the root CD directory
		{
		perror("OpenDir:");
		return;
		}
	// ok, we gonna read its content
	traverse( DirFD, BaseDir );
	closedir(DirFD);
	DirTree.sort();

	//BuildResultTree( 3 );
	//Debug();
}


void KDirTree::traverse( DIR * entrydir, string dirname )
{
	struct dirent *dp;
	DIR * newdir;
	struct stat filestatus;
	
	if ( (entrydir!=NULL) )
		while (	(dp=readdir(entrydir))!=NULL  )
			{
			if ( !IsDir(dirname+"/"+string(dp->d_name)) && (strcmp(dp->d_name,".")&&strcmp(dp->d_name,"..") ) )
				{
				DirTree.push_back(dirname+"/"+string(dp->d_name));
				NumberOfFiles++;
				}
			else
				{
				string newname = dirname+"/"+string(dp->d_name);
				stat( newname.c_str(), &filestatus );
				if ( S_ISDIR(filestatus.st_mode) && strcmp(dp->d_name,".") && strcmp(dp->d_name,"..") && !S_ISLNK(filestatus.st_mode) )
					{
					NumberOfDirectories++;
					newdir = opendir( newname.c_str() );
					traverse(newdir, newname );
					closedir(newdir);
					}
				}
			}
}

int KDirTree::IsDir(string entry)
{
	struct stat filestatus;
	stat(entry.c_str(), &filestatus);
	return S_ISDIR(filestatus.st_mode)|S_ISLNK(filestatus.st_mode);
}

int KDirTree::GetNumberOfTreeEntries()
{
	return ResultTree.size();
}

void KDirTree::BuildResultTree( int level )
{
	LevelToAnalyze = level;
	// erase the previous content
	ClearResultTree();
	// traverse the DirTree list
	for ( std::list<string>::iterator DirTreeEntry = DirTree.begin(); DirTreeEntry!=DirTree.end(); DirTreeEntry++ )
		AddEntryToResult(*DirTreeEntry);
}


TreeNode KDirTree::GetResultTreeEntry( int i )
{
	return ResultTree[i];
}

void KDirTree::ClearResultTree()
{
	ResultTree.erase(ResultTree.begin(), ResultTree.end());
}

void KDirTree::AddEntryToResult(string entry)
{
	ParseString(entry);
	unsigned int niveau;
	for (niveau=0; niveau<ParsedString.size()-1; niveau++)
		{
		if ( !AlreadyStored( ParsedString[niveau], niveau+1 ) )
			ResultTree.push_back( TreeNode( ParsedString[niveau], niveau+1, TYPE_DIR ) );
		if ( (int)niveau >= LevelToAnalyze )
			return;
		}
	if ( !AlreadyStored( ParsedString[niveau], niveau+1 ) )
		ResultTree.push_back( TreeNode( ParsedString[niveau], niveau+1, TYPE_FILE ) );
}

bool KDirTree::AlreadyStored( string str, int level )
{
	for (int j=ResultTree.size()-1; j>=0; j-- )
		if ( (ResultTree[j].Level==level) && (ResultTree[j].FileName == str) )
			return true;
	return false;
}

void KDirTree::ParseString(string entry)
{
	ParsedString.erase( ParsedString.begin(), ParsedString.end() );
	bool cont = true;
	string toparse = entry.substr( BaseDir.size()+1 );
	while ( cont )
		{
		unsigned int pos = toparse.find('/');
		if (pos>=0 && pos<toparse.size())
			{
			ParsedString.push_back(toparse.substr(0,pos));
			toparse = toparse.substr(pos+1);
			}
		else
			{
			ParsedString.push_back(toparse);
			cont = false;
			}
		}
}

void KDirTree::Debug()
{
	for ( unsigned int i=0; i<ResultTree.size(); i++ )
		printf("%d : %s (%s)\n", ResultTree[i].Level, ResultTree[i].FileName.c_str(), ResultTree[i].Type==TYPE_DIR?"DIR":"FILE");
}
